#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <LEDStripe.h>
#include <fade.h>
#include <config.h>
#include <blink.h>

#define REDPIN1 5
#define GREENPIN1 4
#define BLUEPIN1 13

#define REDPIN2 12
#define GREENPIN2 14
#define BLUEPIN2 16


WiFiClient espClient;
PubSubClient client(espClient);


int Sync = 0; //Sync LED2 on the value of LED1

LEDStripe led1 = {0, 0, 5000, 0, 0, 0, 0, 0, 0};
LEDStripe led2 = {0, 0, 5000, 0, 0, 0, 0, 0, 0};

int debugLoop = 0;

void connectWiFi() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.println("Start WiFi Connection");
  int wifiConnectionCount = 0;
  WiFi.begin(SSID, PSK);
  while (WiFi.status() != WL_CONNECTED) {
    delay(5);
    blink_fade();
    Serial.print(".");
    wifiConnectionCount++;
    if(wifiConnectionCount > 1000) {
      Serial.println("No Connection found");
      wifiConnectionCount = 0;
      blink(2);
    }
  }
  digitalWrite(LED_BUILTIN, HIGH);
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}


String getTopicNameString(String topic) {
  //String topicName = String(MQTT_TOPIC_START)+String(topic);
  String topicName = String("led1")+String(topic);
  Serial.println("TopicName: "+topicName);
  return topicName;
}

char* getTopicName(String topic) {
  String topicName = getTopicNameString(topic);
  Serial.println("TopicName: "+topicName);
  int str_len = topicName.length() + 1; 
  char cc[str_len];
  topicName.toCharArray(cc, str_len);
  Serial.println(cc);
  return cc;
}

void connectMQTT() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.print("Reconnecting...");
  if (!client.connect(MQTT_CLIENT_ID, MQTT_USER, MQTT_PASS, getTopicName("/status"), 1, true, "0")) {
    Serial.print("failed, rc=");
    Serial.print(client.state());
    Serial.println(" retrying in 5 seconds");
    digitalWrite(LED_BUILTIN, HIGH);
    delay(2000);
    blink(1);
    delay(3000);
  } 
  digitalWrite(LED_BUILTIN, HIGH);
  Serial.println("Subscribe");
  Serial.println(getTopicName("/#"));
  // @todo rausfinden warum es ein "encoding" like fehler gibt wenn die getTopicName methode benutzt wird.
  //client.subscribe(getTopicName("/#"));
  //client.subscribe("led1/#");
  client.subscribe("#");
  client.publish(getTopicName("/status"), "1", true);
}



//Actions
void pulse_white_three_times() {
  LEDStripe tmpLed();
}


void writeToLedStrips(LEDStripe *l1, LEDStripe *l2) {
  analogWrite(REDPIN1, l1->RED);
  analogWrite(REDPIN2, l2->RED);
  analogWrite(GREENPIN1, l1->GREEN);
  analogWrite(GREENPIN2, l2->GREEN);
  analogWrite(BLUEPIN1, l1->BLUE);
  analogWrite(BLUEPIN2, l2->BLUE);
  debugLoop++;
  if(debugLoop > 50) {
    Serial.println("LED1: "+String(l1->RED)+"|"+String(l1->GREEN)+"|"+String(l1->BLUE));
    Serial.println("LED2: "+String(l2->RED)+"|"+String(l2->GREEN)+"|"+String(l2->BLUE));
    debugLoop = 0;
  }
  
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println("CALLBACK");
  String message = "";
  for (int i=0;i<int(length);i++) {
    message = message + String((char)payload[i]);
  }
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  Serial.print(message);
  Serial.println();
  Serial.println(getTopicNameString("/1/mode"));
  if(String(topic) == getTopicNameString("/1/mode")) {
    if(message == "off") {
      led1.MOD = 0;
    }
    if(message == "fade") {
      led1.MOD = 2;
    }
    if(message == "on") {
      led1.MOD = 1;
    }
  }
  if(String(topic) == getTopicNameString("/2/mode")) {
    if(message == "off") {
      led1.MOD = 0;
    }
    if(message == "fade") {
      led1.MOD = 2;
    }
    if(message == "on") {
      led1.MOD = 1;
    }
  }
  if(String(topic) == getTopicNameString("/1/speed")) {
    led1.SPEED = message.toInt();
  }
  if(String(topic) == getTopicNameString("/2/speed")) {
    led2.SPEED = message.toInt();
  }

  if(String(topic) == getTopicNameString("/1/color")) {
    led1.RED = message.substring(0,3).toInt();
    led1.GREEN = message.substring(3,6).toInt();
    led1.BLUE = message.substring(6,9).toInt();
  }
  if(String(topic) == getTopicNameString("/2/color")) {
    led2.RED = message.substring(0,3).toInt();
    led2.GREEN = message.substring(3,6).toInt();
    led2.BLUE = message.substring(6,9).toInt();
  }
  if(String(topic) ==  getTopicNameString("/sync")) {
    Sync = message.toInt();
  }
  Serial.println("Message done");
}

void prepareMQTT() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.println("Prepare MQTT Client");

  client.setServer(MQTT_BROKER, 1883);
  client.setCallback(callback);
  digitalWrite(LED_BUILTIN, HIGH);
  delay(200);
}

void setup() {
  Serial.begin(115200);
  Serial.println(foo);
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.println();
  Serial.println("Start LED-Controller");
  connectWiFi();
  prepareMQTT();
  connectMQTT();  

  //init LED Strips
  pinMode(REDPIN1, OUTPUT);
  pinMode(REDPIN2, OUTPUT);
  pinMode(GREENPIN1, OUTPUT);
  pinMode(GREENPIN2, OUTPUT);
  pinMode(BLUEPIN1, OUTPUT);
  pinMode(BLUEPIN2, OUTPUT);

  //turn led Strips off
  analogWrite(REDPIN1, 0);
  analogWrite(REDPIN2, 0);
  analogWrite(GREENPIN1, 0);
  analogWrite(GREENPIN2, 0);
  analogWrite(BLUEPIN1, 0);
  analogWrite(BLUEPIN2, 0);
}


void loop() {
  //Serial.println(".");
  if (!client.connected()) {
    connectMQTT();
  }
  client.loop();

  if(led1.MOD == 2) {
    fade(&led1);
  }

  if(Sync == 1) {
    led2.RED = led1.RED;
    led2.GREEN = led1.GREEN;
    led2.BLUE = led1.BLUE;
  } else {
    if(led2.MOD == 2) {
      fade(&led2);
    }
  }

  //Send calculated colors to led-strips
  writeToLedStrips(&led1, &led2);
  delay(10);
}

